#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import logging
from threading import Event, Thread
import time

from driver.DAC5578 import DAC5578, DAC5578_CHANNEL
from driver.ShiftRegister import ShiftRegister

_logger = logging.getLogger(__name__)

ON = True
OFF = False

FlashingState = collections.namedtuple('FlashingState', 'off green red red_green')._make(range(4))


class DigitalAnalogIO(object):
    """!
    Digital/Analog I/O Ansteuerung
    """
    def __init__(self, error_handler):
        """!
        Konstruktor
        Initialisierung der Membervariablen
        @param error_handler: Instanz des Error Handlers
        """
        _logger.debug("DigitalAnalogIO init")
        self._name = "DigitalAnalogIO"
        self._error_handler = error_handler
        self._sr = ShiftRegister()
        self._dac = DAC5578()
        self._stopped = Event()
        self._stopped.clear()
        self._digital_analog_io_thread = None
        self._process_time = 0.1
        self._red_light = False
        self._green_light = False
        self._flash_state = FlashingState.off
        self._flash_speed = 3
        self._blink_timer = time.time() + self._flash_speed     # Timer für Blinken der LEDs
        self._flashing_state = True

        self._fuse_trigger_error_is_set = False

        self.clear_all()

    @property
    def shift_register_bit_string(self):
        """!
        Getter für die ShiftRegister OUT(24bit) und IN(16bit) zurück - getrennt durch ein ';'
        @return Schieberegisterinhalt in Form get_output; get_input
        """
        return "{0:024b}".format(self._sr.get_output()) + "; {0:016b}".format(self._sr.get_input())

    def clear_all(self):
        """!
        Leere das aktuelle Schieberegister
        """
        self._sr.set_output(ShiftRegister.ALL_OFF)
        self._sr.update()

        for ch in DAC5578_CHANNEL:
            self._dac.set_value(channel=ch, value=0)

    # --- thread ---
    def start(self):
        """!
        Startet den Thread für Digital/Analog I/O
        """
        self._stopped.clear()
        self._digital_analog_io_thread = Thread(target=self._process, name=self._name)
        self._digital_analog_io_thread.setDaemon(True)
        self._digital_analog_io_thread.start()

        _logger.debug("DigitalAnalogIO start process")

    def _process(self):
        """!
        Thread zur Ansteuerung der LED (blinken, nicht blinken, an, aus)
        """
        _logger.debug("{} process started".format(self._name))
        last_1s_task_time = time.time()                         # Timer für 1s Task

        while not self._stopped.wait(self._process_time):
            process_start_time = time.time()

            if self._flash_state is FlashingState.red:
                if self._blink_timer < time.time():
                    self._blink_timer = time.time() + self._flash_speed
                    self._led_red_flashing()
            elif self._red_light:
                self._sr.set_output_bit(ShiftRegister.LED_GRN)
                self._sr.clear_output_bit(ShiftRegister.LED_ROT)
            elif self._flash_state is FlashingState.red_green:
                if self._blink_timer < time.time():
                    self._blink_timer = time.time() + self._flash_speed
                    self._led_red_green_flashing()
            elif self._flash_state is FlashingState.green:
                if self._blink_timer < time.time():
                    self._blink_timer = time.time() + self._flash_speed
                    self._led_green_flashing()
            elif self._green_light:
                self._sr.set_output_bit(ShiftRegister.LED_ROT)
                self._sr.clear_output_bit(ShiftRegister.LED_GRN)


            #if self._blink_timer < time.time():
            #    self._blink_timer = time.time() + self._flash_speed
            #    if self._flash_state is FlashingState.green:
            #        self._led_green_flashing()
            #
            #    if self._flash_state is FlashingState.red:
            #        self._led_red_flashing()
            #
            #    if self._flash_state is FlashingState.red_green:
            #        self._led_red_green_flashing()

            # Schieberegister bedienen
            self._sr.update()

            if time.time() - last_1s_task_time >= 1:            # Wenn 1s vergangen ist
                last_1s_task_time = time.time()
                self._do_1s_task()                                   # den 1s Task ausführen

            delta_process_time = time.time() - process_start_time
            wait_time = self._process_time - delta_process_time
            if wait_time <= 0:
                if __debug__:
                    _logger.debug("no wait time:{}".format(delta_process_time))

        _logger.debug("{} process stopped".format(self._name))

    def _do_1s_task(self):
        """!
        Diese Funktion soll im 1s Zyklus aufgerufen werden.
        Hier werden dann nach jeweils 1 Sekunde kurze Task ausgeführt.
        """
        # Überwachung der elektronischen Sicherung
        if self.is_fuse_triggered():        # Hat die Sicherung ausgelöst?
            if not self._fuse_trigger_error_is_set:
                self._error_handler.set_error("F10")        # Ofensteuerung defekt
                _logger.debug("fuse_trigger_error_is_set = True")
                self._fuse_trigger_error_is_set = True
        else:                                               # ansonsten Flag löschen

            self._fuse_trigger_error_is_set = False

    def exit(self):
        """!
        Hauptschalter rot ausschalten und
        Update des Schieberegisters
        """
        self.main_switch_red_flash(OFF)
        self.main_switch_green_flash(OFF)
        self.main_switch_green(ON)
        self._sr.update()
        self._stopped.set()

    # --- combined ---
    def bluetooth(self, value):
        """!
        Schaltet abhängig von value Bluetooth ein/aus
        Wird zum Aktivieren der RS232 Schnittstelle zur GUI benötigt.
        @param value: True - BT ein, False - BT aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.BLE_ON)
            self._sr.set_output_bit(ShiftRegister.RES_BLE)
        else:
            self._sr.clear_output_bit(ShiftRegister.BLE_ON)
            self._sr.clear_output_bit(ShiftRegister.RES_BLE)

    def _main_switch_red(self, value):
        """!
        Hauptschalter rot ein-/ausschalten
        @param value: True - Hauptschalter Rote LED an, False - Hauptschalter Rote LED aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.LED_GRN)
            self._sr.clear_output_bit(ShiftRegister.LED_ROT)
        else:
            self._sr.clear_output_bit(ShiftRegister.LED_GRN)
            self._sr.clear_output_bit(ShiftRegister.LED_ROT)

    def main_switch_red(self, value):
        """!
        Hauptschalter rot ein-/ausschalten
        @param value: True - Hauptschalter Rote LED an, False - Hauptschalter Rote LED aus
        """
        self._green_light = not value
        self._red_light = value

        #if value:
        #    self._sr.set_output_bit(ShiftRegister.LED_GRN)
        #    self._sr.clear_output_bit(ShiftRegister.LED_ROT)
        #else:
        #    self._sr.clear_output_bit(ShiftRegister.LED_GRN)
        #    self._sr.clear_output_bit(ShiftRegister.LED_ROT)

    def _main_switch_green(self, value):
        """!
        Hauptschalter grün ein-/ausschalten
        @param value: True - Hauptschalter Grüne LED an, False - Hauptschalter Grüne LED aus
        """
        if value:
           self._sr.clear_output_bit(ShiftRegister.LED_GRN)
           self._sr.set_output_bit(ShiftRegister.LED_ROT)
        else:
           self._sr.clear_output_bit(ShiftRegister.LED_GRN)
           self._sr.clear_output_bit(ShiftRegister.LED_ROT)

    def main_switch_green(self, value):
        """!
        Hauptschalter grün ein-/ausschalten
        @param value: True - Hauptschalter Grüne LED an, False - Hauptschalter Grüne LED aus
        """
        self._green_light = value
        self._red_light = not value

        #if value:
        #    self._sr.clear_output_bit(ShiftRegister.LED_GRN)
        #    self._sr.set_output_bit(ShiftRegister.LED_ROT)
        #else:
        #    self._sr.clear_output_bit(ShiftRegister.LED_GRN)
        #    self._sr.clear_output_bit(ShiftRegister.LED_ROT)

    def main_switch_red_flash(self, value, speed=3):
        """!
        Hauptschalter rot-blinkend/aus ansteuern
        @param value: True - Hauptschalter Rot Blinken an, False - Hauptschalter Rot Blinken aus
        @param speed: Blinkfrequenz in Hz, Default 3 Hz ('Schnell Blinken')
        """
        # Wenn Blinken aktiv und Frequenz nicht 0 Hz
        if value is ON and speed > 0:
            self._flash_speed = 1 / (2.0 * speed)
            if self._flash_state != FlashingState.red:
                self._flash_state = FlashingState.red
                # Halbe Periodendauer ist die Zeit zum Pinzustand wechseln
                self._blink_timer = time.time() + self._flash_speed
        # sonst nicht Blinken.
        else:
            self._flash_state = FlashingState.off
            self._flash_speed = 0

    def main_switch_green_flash(self, value, speed=3):
        """!
        Hauptschalter grün-blinkend/aus ansteuern
        @param value: True - Hauptschalter Grün Blinken an, False - Hauptschalter Grün Blinken aus
        @param speed: Blinkfrequenz in Hz, Default 3 Hz ('Schnell Blinken')
        """
        # Wenn Blinken aktiv und Frequenz nicht 0 Hz
        if value is ON and speed > 0:
            self._flash_speed = 1 / (2.0 * speed)
            if self._flash_state != FlashingState.green:
                self._flash_state = FlashingState.green
                # Halbe Periodendauer ist die Zeit zum Pinzustand wechseln
                self._blink_timer = time.time() + self._flash_speed
            # sonst nicht Blinken.
        else:
            self._flash_state = FlashingState.off
            self._flash_speed = 0

    def main_switch_red_green_flash(self, value, speed=3):
        """!
        Hauptschalter rot-grün-blinkend/aus ansteuern
        @param value: True - Hauptschalter Rot-Grün Blinken an, False - Hauptschalter Rot-Grün Blinken aus
        @param speed: Blinkfrequenz in Hz, Default 3 Hz ('Schnell Blinken')
        """
        # Wenn Blinken aktiv und Frequenz nicht 0 Hz
        if value is ON and speed > 0:
            self._flash_speed = 1 / (2.0 * speed)
            if self._flash_state != FlashingState.red_green:
                self._flash_state = FlashingState.red_green
                # Halbe Periodendauer ist die Zeit zum Pinzustand wechseln
                self._blink_timer = time.time() + self._flash_speed
            # sonst nicht Blinken.
        else:
            self._flash_state = FlashingState.off
            self._flash_speed = 0

    # --- Analog output ---
    def fan1_set(self, percent):
        """!
        Schaltet den Lüfter1 mit percent Prozent ein
             255 ...  30%
               0 ... 100%
        @param percent: Prozent
        """
        if 0 <= percent <= 100:
            value = 255 - ((255 / 100.0) * percent)
            self._dac.set_value(DAC5578_CHANNEL.F, int(round(value, 0)))
        else:
            _logger.debug("no valid value - [0-100] only allowed")

    def fan2_set(self, percent):
        """!
        Schaltet den Lüfter2 mit percent Prozent ein
             255 ...  30%
               0 ... 100%
        @param percent: Prozent
        """
        if 0 <= percent <= 100:
            value = 255 - ((255 / 100.0) * percent)
            self._dac.set_value(DAC5578_CHANNEL.D, int(round(value, 0)))
        else:
            _logger.debug("no valid value - [0-100] only allowed")

    def fan3_set(self, percent):
        """!
        Schaltet den Lüfter3 mit percent Prozent ein
             255 ...  30%
               0 ... 100%
        @param percent: Prozent
        """
        if 0 <= percent <= 100:
            value = 255 - ((255 / 100.0) * percent)
            self._dac.set_value(DAC5578_CHANNEL.B, int(round(value, 0)))
        else:
            _logger.debug("no valid value - [0-100] only allowed")

    # --- digital output ---
    def res_ble(self, value):
        """!
        Setzt im Schieberegister das Bit für Bluetooth-Reset
        oder setzt das Bit zurück
        @param value: True - BT Reset ein, False - BT Reset aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.RES_BLE)
        else:
            self._sr.clear_output_bit(ShiftRegister.RES_BLE)

    def ble_on(self, value):
        """!
        Setzt im Schieberegister das Bit für Bluetooth-ON
        oder setzt das Bit zurück
        @param value: True - BT ein, False - BT aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.BLE_ON)
        else:
            self._sr.clear_output_bit(ShiftRegister.BLE_ON)

    def v24_on(self, value):
        """!
        Setzt im Schieberegister das Bit für 24V-Stromversorgung
        oder setzt das Bit zurück
        @param value: True - 24V ein, False - 24V aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.V24_ON)
        else:
            self._sr.clear_output_bit(ShiftRegister.V24_ON)

    def enable_battery(self):
        """!
        Verbindet die Batterie zur Netzausfall-Überbrückung mit dem Power Management IC
        """
        self._sr.set_output_bit(ShiftRegister.BAT_ENABLE)

    def disable_battery(self):
        """!
        Trennt die Batterie zur Netzausfall-Überbrückung vom Power Management IC
        """
        self._sr.clear_output_bit(ShiftRegister.BAT_ENABLE)

    def fan1_on(self, value):
        """!
        Setzt im Schieberegister das Bit für Lüfter 1
        oder setzt das Bit zurück
        @param value: True - Lüfter 1 ein, False - Lüfter 1 aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.LUEFTER_ON1)
        else:
            self._sr.clear_output_bit(ShiftRegister.LUEFTER_ON1)

    def fan2_on(self, value):
        """!
        Setzt im Schieberegister das Bit für Lüfter 2
        oder setzt das Bit zurück
        @param value: True - Lüfter 2 ein, False - Lüfter 2 aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.LUEFTER_ON2)
        else:
            self._sr.clear_output_bit(ShiftRegister.LUEFTER_ON2)

    def fan3_on(self, value):
        """!
        Setzt im Schieberegister das Bit für Lüfter 3
        oder setzt das Bit zurück
        @param value: True - Lüfter 3 ein, False - Lüfter 3 aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.LUEFTER_ON3)
        else:
            self._sr.clear_output_bit(ShiftRegister.LUEFTER_ON3)

    def led_yellow(self, value):
        """!
        Setzt im Schieberegister das Bit für die gelbe LED
        oder setzt das Bit zurück
        @param value: True - Gelbe LED ein, False - Gelbe LED aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.LED_YELLOW)
        else:
            self._sr.clear_output_bit(ShiftRegister.LED_YELLOW)

    def led_blue(self, value):
        """!
        Setzt im Schieberegister das Bit für die blaue LED
        oder setzt das Bit zurück
        @param value: True - blaue LED ein, False - blaue LED aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.LED_BLUE)
        else:
            self._sr.clear_output_bit(ShiftRegister.LED_BLUE)

    def press_on(self, value):
        """!
        Setzt im Schieberegister das Bit für den Press-Stempel Motor
        oder setzt das Bit zurück
        @param value: True - Press-Stempel Motor an, False - Press-Stempel Motor aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.PRESS_ON)
        else:
            self._sr.clear_output_bit(ShiftRegister.PRESS_ON)

    def press_dir(self, value):
        """!
        Setzt im Schieberegister das Bit für die Richtung des Press-Stempel Motors
        oder setzt das Bit zurück
        @param value: True - Press-Stempel Motor vorwärts, False - Press-Stempel Motor rückwärts
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.PRESS_DIR)
        else:
            self._sr.clear_output_bit(ShiftRegister.PRESS_DIR)

    def lift_auf(self):
        """!
        Lässt den Türlift nach unten fahren (Türe auf)
        """
        self._sr.set_output_bit(ShiftRegister.LIFT_AB)
        self._sr.clear_output_bit(ShiftRegister.LIFT_AUF)

    def lift_zu(self):
        """!
        Lässt den Türlift nach oben fahren (Türe zu)
        """
        self._sr.set_output_bit(ShiftRegister.LIFT_AUF)
        self._sr.clear_output_bit(ShiftRegister.LIFT_AB)

    def lift_stop(self):
        """!
        Stoppt den Türlift
        """
        self._sr.clear_output_bit(ShiftRegister.LIFT_AB)
        self._sr.clear_output_bit(ShiftRegister.LIFT_AUF)

    def rel_res(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 1
        oder setzt das Bit zurück
        @param value: True - Reserve Relais ein, False - Reserve Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT1)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT1)

    def rel_vakuum(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 2 (Vakuum)
        oder setzt das Bit zurück
        @param value: True - Vakuum Pumpe Relais ein, False - Vakuum Pumpe Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT2)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT2)

    def v24_out_res1(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 3 (24V-Spannung)
        oder setzt das Bit zurück
        @param value: True - Reserve Relais 1 ein, False - Reserve Relais 1 aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT3)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT3)

    def valve_argon(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 4 (Argon-Ventil)
        oder setzt das Bit zurück
        @param value: True - Argon Relais ein, False - Argon Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT4)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT4)

    def valve_vacuum(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 5 (Vakuum-Ventil)
        oder setzt das Bit zurück
        @param value: True - Vakuum Ventil Relais ein, False - Vakuum Ventil Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT5)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT5)

    def valve_forced_cooling(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 6 (Zwangskühlung-Ventil)
        oder setzt das Bit zurück
        @param value: True - Zwangskühlung-Ventil Relais ein, False - Zwangskühlung-Ventil Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT6)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT6)

    def valve_vent(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 7 (Entlüftungs-Ventil)
        oder setzt das Bit zurück
        @param value: True - Entlüftungs-Ventil Relais ein, False - Entlüftungs-Ventil Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT7)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT7)

    def valve_safety_on(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 8 (Sicherheitsverriegelung)
        oder setzt das Bit zurück
        @param value: True - Sicherheitsverriegelung Relais ein, False - Sicherheitsverriegelung Relais aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT8)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT8)

    def v24_out_res2(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 9
        oder setzt das Bit zurück
        @param value: True - Reserve Relais 2 ein, False - Reserve Relais 2 aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT9)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT9)

    def v24_out_res3(self, value):
        """!
        Setzt im Schieberegister das Bit für Digital OUT 10
        oder setzt das Bit zurück
        @param value: True - Reserve Relais 3 ein, False - Reserve Relais 3 aus
        """
        if value:
            self._sr.set_output_bit(ShiftRegister.DOUT10)
        else:
            self._sr.clear_output_bit(ShiftRegister.DOUT10)

    # --- digital input ---
    def main_switch_is_set(self):
        """!
        Lese aktuellen Wert des Hauptschalters im Schieberegister
        @return True - Haupttaster betätigt, False - Haupttaster nicht betätigt
        """
        if self._sr.input_is_set(ShiftRegister.MAIN_SWITCH_IN):
            return not True
        else:
            return not False

    def door_end_switch_open_is_set(self):
        """!
        Lese aktuellen Wert des Tür-Offen-Endschalters im Schieberegister
        @return True - Endschalter betätigt, False - Endschalter nicht betätigt
        """
        if self._sr.input_is_set(ShiftRegister.DOOR_END_SWITCH_OPEN):
            return True
        else:
            return False

    def door_end_switch_close_is_set(self):
        """!
        Lese aktuellen Wert des Tür-Geschlossen-Endschalters im Schieberegister
        @return True - Endschalter betätigt, False - Endschalter nicht betätigt
        """
        if self._sr.input_is_set(ShiftRegister.DOOR_END_SWITCH_CLOSED):
            return True
        else:
            return False

    @staticmethod
    def is_din0():
        # if self._sr.input_is_set(ShiftRegister.DIN0):
        #     return not False
        # else:
        #     return not True
        _logger.debug("is_din0 not implemented")
        return False

    def is_safety_lock(self):
        """!
        Lese aktuellen Wert des Sicherheitsschalters im Schieberegister
        @return True - Sicherheitsschalters betätigt, False - Sicherheitsschalters nicht betätigt
        """
        if self._sr.input_is_set(ShiftRegister.SAFETY_SWITCH):
            return not False
        else:
            return not True

    @staticmethod
    def is_stemp_hall_a():
        # if self._sr.input_is_set(ShiftRegister.):
        #     return not False
        # else:
        #     return not True
        _logger.debug("is_din0 not implemented")
        return False

    @staticmethod
    def is_stem_hall_b():
        # if self._sr.input_is_set(ShiftRegister.):
        #     return not False
        # else:
        #     return not True
        _logger.debug("is_din0 not implemented")
        return False

    def is_fan1_running(self):
        """!
        Lese aktuellen Wert des Lüfter 1 im Schieberegister
        @return True - Lüfter dreht sich, False - Lüfter dreht sich nicht
        """
        if self._sr.input_is_set(ShiftRegister.FAN1_ON):
            return True
        else:
            return False

    def is_fan2_running(self):
        """!
        Lese aktuellen Wert des Lüfter 2 im Schieberegister
        @return True - Lüfter dreht sich, False - Lüfter dreht sich nicht
        """
        if self._sr.input_is_set(ShiftRegister.FAN2_ON):
            return True
        else:
            return False

    def is_fan3_running(self):
        """!
        Lese aktuellen Wert des Lüfter 3 im Schieberegister
        @return True - Lüfter dreht sich, False - Lüfter dreht sich nicht
        """
        if self._sr.input_is_set(ShiftRegister.FAN3_ON):
            return True
        else:
            return False

    @staticmethod
    def is_di_res1():
        # if self._sr.input_is_set(ShiftRegister.):
        #     return not False
        # else:
        #     return not True
        _logger.debug("is_din0 not implemented")
        return False

    @staticmethod
    def is_di_res2():
        # if self._sr.input_is_set(ShiftRegister.):
        #     return not False
        # else:
        #     return not True
        _logger.debug("is_din0 not implemented")
        return False

    @staticmethod
    def is_di_res3():
        # if self._sr.input_is_set(ShiftRegister.):
        #     return not False
        # else:
        #     return not True
        _logger.debug("is_din0 not implemented")
        return False

    def is_fuse_triggered(self):
        """!
        Zustand der elektronischen Sicherung / 24V Versorgung.
        @return True - Sicherung hat ausgelöst, False - Sicherung hat nicht ausgelöst
        """
        if self._sr.input_is_set(ShiftRegister.V24_STATUS):     # Ist V24_STATUS auf H / hat die Sicherung ausgelöst?
            return True                                         # Sicherung hat ausgelöst
        else:
            return False                                        # Sicherung hat nicht ausgelöst

    # --- flash ---
    def _led_green_flashing(self):
        """!
        Togglen der grünen LED
        """
        if self._flashing_state:
            self._main_switch_green(ON)
        else:
            self._main_switch_green(OFF)
        # invert for next call
        self._flashing_state = not self._flashing_state

    def _led_red_flashing(self):
        """!
        Togglen der roten LED
        """
        if self._flashing_state:
            self._main_switch_red(ON)
        else:
            self._main_switch_red(OFF)
        # invert for next call
        self._flashing_state = not self._flashing_state

    def _led_red_green_flashing(self):
        """!
        Togglen der roten LED
        """
        if self._flashing_state:
            self._main_switch_red(ON)
        else:
            self._main_switch_green(ON)
        # invert for next call
        self._flashing_state = not self._flashing_state
