#!/usr/bin/env python
# -*- coding: utf-8 -*-
import ctypes
import logging
import sys
import time

import mraa
from upm.pyupm_ad7124 import *

sys.path.append('/home/root/gs1')
sys.path.append('/home/root/gs1/src')
sys.path.append('/home/root/gs1/src/psbp')

AD7124_CFG_REG_BURNOUT = lambda x: ((x & 0x3) << 9)

# def AD7124_CFG_REG_BURNOUT(x):
#    return (x & 0x3) << 9

AD7124_CFG_REG_REF_SEL = lambda x: (x & 0x3) << 3
AD7124_CFG_REG_PGA = lambda x: ((x & 0x7) << 0)

AD7124_FILT_REG_FILTER = lambda x: (ctypes.c_uint32((x & 0x7) << 21)).value
AD7124_FILT_REG_POST_FILTER = lambda x: (ctypes.c_uint32((x & 0x7) << 17)).value
AD7124_FILT_REG_FS = lambda x: (ctypes.c_uint32((x & 0x7FF) << 0)).value
AD7124_FILT_REG_SINGLE_CYCLE = 1 << 16

AD7124_CH_MAP_REG_SETUP = lambda x: ((x & 0x7) << 12)
AD7124_CH_MAP_REG_AINP = lambda x: ((x & 0x1F) << 5)
AD7124_CH_MAP_REG_AINM = lambda x: ((x & 0x1F) << 0)

AD7124_IO_CTRL1_REG_IOUT1 = lambda x: ((x & 0x7) << 11)
AD7124_IO_CTRL1_REG_IOUT0 = lambda x: ((x & 0x7) << 8)
AD7124_IO_CTRL1_REG_IOUT_CH1 = lambda x: ((x & 0xF) << 4)
AD7124_IO_CTRL1_REG_IOUT_CH0 = lambda x: ((x & 0xF) << 0)

AD7124_COMM_REG_RA = lambda x: (x & 0x3F)
AD7124_STATUS_REG_CH_ACTIVE = lambda x: (x & 0xF)
AD7124_ADC_CTRL_REG_POWER_MODE = lambda x: ((x & 0x3) << 6)
AD7124_ADC_CTRL_REG_MODE = lambda x: ((x & 0xF) << 2)
AD7124_ADC_CTRL_REG_CLK_SEL = lambda x: ((x & 0x3) << 0)

# Konstanten:
ENABLE_AD_LOGGING = False

# table degree Kelvin, uV
table_thermocouple_S = [
    [273, 283, 293, 313, 333, 353, 373, 403, 433, 473,
     523, 573, 623, 673, 723, 773, 873, 973, 1073, 1173,
     1273, 1373, 1473, 1573, 1673, 1773, 1873, 1973, 2033, 2273],
    [0, 55, 113, 235, 365, 502, 646, 872, 1110, 1441,
     1874, 2323, 2786, 3259, 3742, 4233, 5239, 6275, 7345, 8449,
     9587, 10757, 11951, 13159, 14373, 15582, 16777, 17947, 18609, 21489]]
# table degree Kelvin, uV
table_thermocouple_K = [[173, 223, 253, 273, 293, 323, 373, 423, 473, 523, 573, 623, 673, 1273],
                        [-3554, -1889, -778, 0, 833, 2058, 4131, 6174, 8174, 10189, 12244, 14328, 16432, 41311]]
# table degree Kelvin, resistance in ohm
table_KTY = [[223, 233, 243, 253, 263, 273, 283, 293, 298, 303, 313, 323, 333, 343, 353, 363, 373, 383, 393],
             [515, 567, 624, 684, 747, 815, 886, 961, 1000, 1040, 1122, 1209, 1299, 1392, 1490, 1591, 1696, 1805, 1915]]

# global ADC_CTRL_REG_value

_logger = logging.getLogger(__name__)


def interpolation_temp(table, x_value):  # measure value => degree K
    i = len(table[1])
    if i == len(table[0]):  # table ok?
        i = i - 1
        if x_value > table[1][i]:  # value to high, error
            return 0.0
        while x_value < table[1][i]:
            if i == 0:
                return 0.0  # value to low, error
            i = i - 1
        slope = (table[0][i + 1] - table[0][i]) * 1.0 / (table[1][i + 1] - table[1][i]) * 1.0
        return table[0][i] + (x_value - table[1][i]) * slope
    else:
        return 0.0  # table lengths different


def interpolation_temp_reverse(table, y_value):  # degree K => measure
    i = len(table[0])
    if i == len(table[1]):  # table ok?
        i = i - 1
        if y_value > table[0][i]:  # value to high, error
            return 0.0
        while y_value < table[0][i]:
            if i == 0:
                return 0.0  # value to low, error
            i = i - 1
        slope = (table[1][i + 1] - table[1][i]) * 1.0 / (table[0][i + 1] - table[0][i]) * 1.0
        return table[1][i] + (y_value - table[0][i]) * slope
    else:
        return 0.0  # table lengths different


degree_sign = u'\N{DEGREE SIGN}'


class Ad7124(object):
    channel_offset = 8388608.0  # = 0x800000
    adc_val_sens_break = 5500000  # threshold for break detection
    err_thermocouple_1 = 1
    err_thermocouple_3 = 3

    def __init__(self):
        # Variable der Ausgabewerte:
        self._temp_elo = 273.0
        self._thermocouple_1 = 273.0
        self._thermocouple_3 = 273.0
        self._pressure_sensor_1 = 0.0
        self._pressure_sensor_2 = 0.0
        self._pressure_sensor_3 = 0.0

        # Variable intern:
        self._compensation_typS = 0.0
        self._compensation_typK = 0.0

        self._ad7124 = AD7124(0)
        _logger.debug("AD7124 Initialization")
        _logger.debug("Device ID: 0x{:x}".format(self._ad7124.ReadDeviceRegister(self._ad7124.AD7124_ID)))
        _logger.debug("Device Status Register: {}".format(self._ad7124.ReadDeviceRegister(self._ad7124.AD7124_Status)))

        # enable ref voltage for DAC5578
        regnr = self._ad7124.AD7124_ADC_Control
        setvalue = self._ad7124.ReadDeviceRegister(regnr)
        setvalue |= AD7124_ADC_CTRL_REG_REF_EN
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)
        setvalue = self._ad7124.ReadDeviceRegister(regnr)
        _logger.debug("Ref voltage enabled: {}".format(setvalue))
        _logger.debug("Device Status Register: {}".format(self._ad7124.ReadDeviceRegister(self._ad7124.AD7124_Status)))

        # thermocouple 1 typ S
        _logger.debug("Config for Thermocouple 1:")
        regnr = self._ad7124.AD7124_Channel_0
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(0)  # select setup0
        setvalue |= AD7124_CH_MAP_REG_AINP(7)  # set AIN7
        setvalue |= AD7124_CH_MAP_REG_AINM(8)  # set AIN8
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_0: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # thermocouple 3 typ K
        _logger.debug("Config for Thermocouple 3:")
        regnr = self._ad7124.AD7124_Channel_1
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(1)  # select setup1
        setvalue |= AD7124_CH_MAP_REG_AINP(1)  # set AIN1
        setvalue |= AD7124_CH_MAP_REG_AINM(0)  # set AIN0
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_1: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # kty82/110
        _logger.debug("Config for KTY82:")
        regnr = self._ad7124.AD7124_Channel_2
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(2)  # select setup2
        setvalue |= AD7124_CH_MAP_REG_AINP(3)  # set AIN3
        setvalue |= AD7124_CH_MAP_REG_AINM(4)  # set AIN4
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_2: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # check thermocouple 1
        _logger.debug("Config check thermocouple 1:")
        regnr = self._ad7124.AD7124_Channel_3
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(3)  # select setup3
        setvalue |= AD7124_CH_MAP_REG_AINP(7)  # set AIN7
        setvalue |= AD7124_CH_MAP_REG_AINM(8)  # set AIN8
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_3: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # check thermocouple 3
        _logger.debug("Config check thermocouple 3:")
        regnr = self._ad7124.AD7124_Channel_4
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(3)  # select setup3
        setvalue |= AD7124_CH_MAP_REG_AINP(1)  # set AIN1
        setvalue |= AD7124_CH_MAP_REG_AINM(0)  # set AIN0
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_4: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # pressure sensor 1 (150psi)
        _logger.debug("Pressure sensor 1 (150psi):")
        regnr = self._ad7124.AD7124_Channel_5
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(4)  # select setup4
        setvalue |= AD7124_CH_MAP_REG_AINP(9)   # set AIN9
        setvalue |= AD7124_CH_MAP_REG_AINM(10)  # set AIN10
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_5: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # pressure sensor 2 (60psi)
        _logger.debug("Pressure sensor 2 (60psi):")
        regnr = self._ad7124.AD7124_Channel_6
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(5)  # select setup5
        setvalue |= AD7124_CH_MAP_REG_AINP(12)  # set AIN12
        setvalue |= AD7124_CH_MAP_REG_AINM(11)  # set AIN11
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_6: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # pressure sensor 3 (1.6bar)
        _logger.debug("Pressure sensor 3 (1.6bar):")
        regnr = self._ad7124.AD7124_Channel_7
        setvalue = 0
        # setvalue |= AD7124_CH_MAP_REG_CH_ENABLE     # enable channel
        setvalue |= AD7124_CH_MAP_REG_SETUP(6)  # select setup6
        setvalue |= AD7124_CH_MAP_REG_AINP(14)  # set AIN14
        setvalue |= AD7124_CH_MAP_REG_AINM(13)  # set AIN13
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Channel_7: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 0 thermocouple typ S
        regnr = self._ad7124.AD7124_Config_0
        setvalue = 0
        setvalue |= AD7124_CFG_REG_BIPOLAR  # select bipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(0)  # burnout current source off
        setvalue |= AD7124_CFG_REG_AIN_BUFP  # analog input buffer on
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_BUFM  # reference buffer on
        setvalue |= AD7124_CFG_REG_REF_BUFP
        setvalue |= AD7124_CFG_REG_REF_SEL(2)  # internal reference
        setvalue |= AD7124_CFG_REG_PGA(6)  # GAIN64
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_0: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 1 thermocouple typ K
        regnr = self._ad7124.AD7124_Config_1
        setvalue = 0
        setvalue |= AD7124_CFG_REG_BIPOLAR  # select bipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(0)  # burnout current source off
        setvalue |= AD7124_CFG_REG_AIN_BUFP  # analog input buffer on
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_BUFM  # reference buffer on
        setvalue |= AD7124_CFG_REG_REF_BUFP
        setvalue |= AD7124_CFG_REG_REF_SEL(2)  # internal reference
        setvalue |= AD7124_CFG_REG_PGA(5)  # GAIN32
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_1: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 2 sensor KTY
        regnr = self._ad7124.AD7124_Config_2
        setvalue = 0
        # setvalue |= AD7124_CFG_REG_BIPOLAR   # unipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(0)  # burnout current source off
        setvalue |= AD7124_CFG_REG_REF_BUFP    # reference buffer on
        setvalue |= AD7124_CFG_REG_REF_BUFM
        setvalue |= AD7124_CFG_REG_AIN_BUFP    # analog input buffer on
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_SEL(0)  # use REFIN1 as reference
        setvalue |= AD7124_CFG_REG_PGA(0)  # GAIN1
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_2: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 3 thermocouple sensor break check
        regnr = self._ad7124.AD7124_Config_3
        setvalue = 0
        setvalue |= AD7124_CFG_REG_BIPOLAR     # bipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(3)  # burnout current source 4uA
        setvalue |= AD7124_CFG_REG_REF_BUFP
        setvalue |= AD7124_CFG_REG_REF_BUFM
        setvalue |= AD7124_CFG_REG_AIN_BUFP
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_SEL(2)  # internal reference
        setvalue |= AD7124_CFG_REG_PGA(4)  # GAIN 16
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_3: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 4 pressure sensor 1 150psi
        regnr = self._ad7124.AD7124_Config_4
        setvalue = 0
        setvalue |= AD7124_CFG_REG_BIPOLAR     # bipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(0)  # burnout current source off
        setvalue |= AD7124_CFG_REG_REF_BUFP    # reference buffer on
        setvalue |= AD7124_CFG_REG_REF_BUFM
        setvalue |= AD7124_CFG_REG_AIN_BUFP    # analog input buffer on
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_SEL(2)  # internal reference
        setvalue |= AD7124_CFG_REG_PGA(5)      # GAIN32
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_4: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 5 pressure sensor 2 60psi
        regnr = self._ad7124.AD7124_Config_5
        setvalue = 0
        setvalue |= AD7124_CFG_REG_BIPOLAR     # bipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(0)  # burnout current source off
        setvalue |= AD7124_CFG_REG_REF_BUFP    # reference buffer on
        setvalue |= AD7124_CFG_REG_REF_BUFM
        setvalue |= AD7124_CFG_REG_AIN_BUFP    # analog input buffer on
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_SEL(2)  # internal reference
        setvalue |= AD7124_CFG_REG_PGA(6)      # GAIN64
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_5: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set up 6pressure sensor 3 1.6bar
        regnr = self._ad7124.AD7124_Config_6
        setvalue = 0
        setvalue |= AD7124_CFG_REG_BIPOLAR     # bipolar operation
        setvalue |= AD7124_CFG_REG_BURNOUT(0)  # burnout current source off
        setvalue |= AD7124_CFG_REG_REF_BUFP    # reference buffer on
        setvalue |= AD7124_CFG_REG_REF_BUFM
        setvalue |= AD7124_CFG_REG_AIN_BUFP    # analog input buffer on
        setvalue |= AD7124_CFG_REG_AINN_BUFM
        setvalue |= AD7124_CFG_REG_REF_SEL(2)  # internal reference
        setvalue |= AD7124_CFG_REG_PGA(5)      # GAIN32
        setvalue &= 0xFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Config_6: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 0
        regnr = self._ad7124.AD7124_Filter_0
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS, 1920 => 10SPS
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_0: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 1
        regnr = self._ad7124.AD7124_Filter_1
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_1: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 2
        regnr = self._ad7124.AD7124_Filter_2
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS
        setvalue |= AD7124_FILT_REG_SINGLE_CYCLE
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_2: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 3
        regnr = self._ad7124.AD7124_Filter_3
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_3: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 4
        regnr = self._ad7124.AD7124_Filter_4
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_4: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 5
        regnr = self._ad7124.AD7124_Filter_5
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_5: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set filter 6
        regnr = self._ad7124.AD7124_Filter_6
        setvalue = 0
        setvalue |= AD7124_FILT_REG_POST_FILTER(3)
        setvalue |= AD7124_FILT_REG_FILTER(0)  # set SINC4
        setvalue |= AD7124_FILT_REG_FS(384)  # FS = 384 => 50 SPS
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("Filter_6: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set IO_Control_1
        # Konstantstromquelle für KTY82 einschalten (250uA)
        regnr = self._ad7124.AD7124_IOCon1  # select IO_Control_1 register
        setvalue = 0
        # setvalue = self._ad7124.ReadDeviceRegister(regnr)
        setvalue |= AD7124_IO_CTRL1_REG_IOUT0(3)  # set IOUT0 current to 250uA
        setvalue |= AD7124_IO_CTRL1_REG_IOUT_CH0(2)  # set channel AIN2
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("IO_Control_1: {:024b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        # set IO_Control_2
        # BIAS-Spannung für Thermoelemente einschalten
        regnr = self._ad7124.AD7124_IOCon2  # select IO_Control_2 register
        setvalue = 0
        setvalue |= AD7124_8_IO_CTRL2_REG_GPIO_VBIAS8
        setvalue |= AD7124_8_IO_CTRL2_REG_GPIO_VBIAS5
        setvalue |= AD7124_8_IO_CTRL2_REG_GPIO_VBIAS0
        setvalue &= 0xFFFFFF
        self._ad7124.WriteDeviceRegister(regnr, setvalue)  # write data to ADC
        _logger.debug("IO_Control_2: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))

        ####################################################################################################################
        # ADC control register: full power mode, enable reference, single conversion mode
        regnr = self._ad7124.AD7124_ADC_Control
        setvalue = 0
        setvalue |= AD7124_ADC_CTRL_REG_REF_EN  # enable internal reference
        setvalue |= AD7124_ADC_CTRL_REG_POWER_MODE(2)  # full power = 2
        # setvalue |= AD7124_ADC_CTRL_REG_DOUT_RDY_DEL
        setvalue |= AD7124_ADC_CTRL_REG_DATA_STATUS  # status register is sent after each data register read
        setvalue |= AD7124_ADC_CTRL_REG_CS_EN
        setvalue |= AD7124_ADC_CTRL_REG_MODE(1)  # single conversation mode
        setvalue &= 0xFFFF

        global ADC_CTRL_REG_value
        ADC_CTRL_REG_value = setvalue
        self._ad7124.WriteDeviceRegister(regnr, ADC_CTRL_REG_value)
        _logger.debug("Adc Control Register: 0x{:016b}".format(self._ad7124.ReadDeviceRegister(regnr)))
        # _logger.debug("Device Status Register: 0x{:02x}".format(self._ad7124.ReadDeviceRegister(self._ad7124.AD7124_Status)))

        self.kalibrate_channel(self._ad7124.AD7124_Channel_0)  # thermocouple typ S
        self.kalibrate_channel(self._ad7124.AD7124_Channel_1)  # thermocouple typ K
        ####################################################################################################################

    @property
    def dev_ad7124(self):
        return self._ad7124

    def kalibrate_channel(self, ch):
        my_adc = self._ad7124

        my_reg_nr = my_adc.AD7124_ADC_Control
        self.enable_channel(ch)
        time.sleep(0.1)
        _logger.debug("channel {}".format(ch - 9))
        _logger.debug("gain old {}".format(my_adc.ReadDeviceRegister(40 + ch)))  # my_adc.AD7124_Offset
        _logger.debug("offset old {}".format(my_adc.ReadDeviceRegister(32 + ch)))  # my_adc.AD7124_Offset
        my_adc.WriteDeviceRegister(32 + ch, 0x800000)  # reset offset
        my_set_value = ADC_CTRL_REG_value
        my_set_value &= ~0x00C0  # low power mode (00)
        my_set_value |= 0x0040  # medium power mode (01)
        my_set_value &= ctypes.c_uint32(~AD7124_ADC_CTRL_REG_MODE(15)).value
        my_set_value |= ctypes.c_uint32(AD7124_ADC_CTRL_REG_MODE(6)).value  # full scale calibration
        my_set_value &= 0xFFFF
        my_adc.WriteDeviceRegister(my_reg_nr, my_set_value)  # write data to ADC
        time.sleep(0.1)  # calibration is running
        if my_adc.WaitForConvReady(10000) == -3:
            _logger.debug("TIMEOUT")
        my_set_value = ADC_CTRL_REG_value
        my_set_value &= ~0x00C0  # low power mode (00)
        my_set_value |= 0x0040  # medium power mode (01)
        my_set_value &= ctypes.c_uint16(~AD7124_ADC_CTRL_REG_MODE(15)).value
        my_set_value |= ctypes.c_uint16(AD7124_ADC_CTRL_REG_MODE(5)).value  # offset calibration
        my_set_value &= 0xFFFF
        my_adc.WriteDeviceRegister(my_reg_nr, my_set_value)  # write data to ADC
        time.sleep(0.1)  # calibration is running
        if my_adc.WaitForConvReady(10000) == -3:
            _logger.debug("TIMEOUT")
        my_adc.WriteDeviceRegister(my_reg_nr, ADC_CTRL_REG_value)  # reset ADC control
        time.sleep(0.1)
        _logger.debug("gain cal {}".format(my_adc.ReadDeviceRegister(40 + ch)))  # my_adc.AD7124_Offset
        _logger.debug("offset cal {}".format(my_adc.ReadDeviceRegister(32 + ch)))  # my_adc.AD7124_Offset
        self.disable_channel(ch)

    def enable_channel(self, channel):
        my_adc = self._ad7124
        my_reg_nr = channel  # select ADC_Control register
        my_set_value = my_adc.ReadDeviceRegister(my_reg_nr)
        my_set_value |= ctypes.c_uint32(AD7124_CH_MAP_REG_CH_ENABLE).value  # enable channel
        my_set_value &= 0xFFFF
        my_adc.WriteDeviceRegister(my_reg_nr, my_set_value)  # write data to ADC

    def disable_channel(self, channel):
        my_adc = self._ad7124
        my_reg_nr = channel  # select ADC_Control register
        my_set_value = my_adc.ReadDeviceRegister(my_reg_nr)
        my_set_value &= ctypes.c_uint32(~AD7124_CH_MAP_REG_CH_ENABLE).value  # disable channel
        my_set_value &= 0xFFFF
        my_adc.WriteDeviceRegister(my_reg_nr, my_set_value)  # write data to ADC

    def start_single_conversion(self):
        my_adc = self._ad7124
        my_reg_nr = my_adc.AD7124_ADC_Control  # select ADC_Control register
        #   my_value = 0x584
        my_value = ADC_CTRL_REG_value  # 0x784
        my_adc.WriteDeviceRegister(my_reg_nr, my_value)  # write data to ADC

    def read_channel(self, ch):
        my_adc = self._ad7124
        self.enable_channel(ch)

        start = time.time()
        self.start_single_conversion()
        time.sleep(0.1)
        if my_adc.WaitForConvReady(10000) == -3:
            _logger.debug("TIMEOUT")
            return 0
        datap = new_intPointer()
        my_adc.ReadData(datap)
        end = time.time()
        if ENABLE_AD_LOGGING:
            _logger.debug("read channel time: {}".format(end - start))
        self.disable_channel(ch)
        data = intPointer_value(datap)
        return data

    def read_thermocouple_one(self):
        adc = self._ad7124
        regnr = adc.AD7124_Channel_0
        adc_val = self.read_channel(regnr)
        adc_voltage = 2.5 * ((adc_val - Ad7124.channel_offset) / (Ad7124.channel_offset * 64.0)) * 1000000  # in uV
        adc_voltage = adc_voltage + self._compensation_typS  # cold junction compensation
        meas_val = interpolation_temp(table_thermocouple_S, adc_voltage)
        self._thermocouple_1 = round(meas_val, 1)  # round_temp(self._thermocouple_1, meas_val)
        return self._thermocouple_1

    def read_thermocouple_three(self):
        adc = self._ad7124
        regnr = adc.AD7124_Channel_1
        adc_val = self.read_channel(regnr)
        adc_voltage = 2.5 * ((adc_val - Ad7124.channel_offset) / (Ad7124.channel_offset * 32.0)) * 1000000  # in uV
        adc_voltage = adc_voltage + self._compensation_typK  # cold junction compensation
        meas_val = interpolation_temp(table_thermocouple_K, adc_voltage)
        self._thermocouple_3 = round(meas_val, 1)  # round_temp(self._thermocouple_3, meas_val)
        return self._thermocouple_3

    def read_cold_junction(self):
        adc = self._ad7124
        regnr = adc.AD7124_Channel_2  # KTY
        adc_val = self.read_channel(regnr)
        r_rtd = 2000 / 16777216.0 * adc_val  # 2kOhm = max value (16777216)
        meas_val = interpolation_temp(table_KTY, r_rtd)
        self._temp_elo = round(meas_val, 1)  # round_temp(self._temp_elo, meas_val)

        if self._temp_elo > 0:  # measure ok
            self._compensation_typS = interpolation_temp_reverse(table_thermocouple_S, self._temp_elo)
            self._compensation_typK = interpolation_temp_reverse(table_thermocouple_K, self._temp_elo)

        return self._temp_elo

    def read_pressure_sensor_one(self):
        adc = self._ad7124
        regnr = adc.AD7124_Channel_5
        adc_val = self.read_channel(regnr)
        adc_voltage = 2.5 * ((adc_val - Ad7124.channel_offset) / (Ad7124.channel_offset * 32.0)) * 1000  # in mV
        self._pressure_sensor_1 = (adc_voltage / 45.75) * 150.0 * 68.9476   # 45.75mV @150psi, 1psi = 68,9476mbar
        return round(self._pressure_sensor_1, 1)    # Runden auf 1 Kommastelle

    def read_pressure_sensor_two(self):
        adc = self._ad7124
        regnr = adc.AD7124_Channel_6
        adc_val = self.read_channel(regnr)
        adc_voltage = 2.5 * ((adc_val - Ad7124.channel_offset) / (Ad7124.channel_offset * 64.0)) * 1000  # in mV
        self._pressure_sensor_2 = (adc_voltage / 30.0) * 60.0 * 68.9476   # 30mV @60psi, 1psi = 68,9476mbar
        return round(self._pressure_sensor_2, 1)    # Runden auf 1 Kommastelle

    def read_pressure_sensor_three(self):
        adc = self._ad7124
        regnr = adc.AD7124_Channel_7
        adc_val = self.read_channel(regnr)
        adc_voltage = 2.5 * ((adc_val - Ad7124.channel_offset) / (Ad7124.channel_offset * 32.0)) * 1000  # in mV
        self._pressure_sensor_3 = (adc_voltage / 40.75) * 1.6 * 1000   # 40.75mV @1.6bar, 1bar = 1000mbar
        return round(self._pressure_sensor_3, 1)    # Runden auf 1 Kommastelle
